#!/usr/bin/env python3

"""
Contains free functions for getting/setting various VRROOM modes.
"""

from enum import IntEnum
import socket
from . import get_command_base, set_command_base
from .enums import OnOffSwitch


class OperationMode(IntEnum):
    """
    Enumerates the operation modes of the VRROOM switch.
    """

    SPLITTER_VRR = 0
    SPLITTER_UPSCALE = 1
    MATRIX_TMDS = 2
    MATRIX_TMDS_DOWNSCALE = 3
    MATRIX_FRL5_TMDS = 4

    def __str__(self):
        return str(self.value)

    @staticmethod
    def from_string(string: str) -> "OperationMode":
        """
        Converts a string into a value of this enumeration, if possible.
        """
        return OperationMode(int(string))

    @staticmethod
    def pattern() -> str:
        """
        Returns a regex pattern that matches values of this enumeration.
        """
        return f"[{OperationMode.SPLITTER_VRR}-{OperationMode.MATRIX_FRL5_TMDS}]"


__TARGET_MODE_OPERATION = "opmode"
__VALUE_CONVERTERS_MODE_OPERATION = [OperationMode.from_string]
__VALUE_PATTERNS_MODE_OPERATION = [OperationMode.pattern()]


def get_operation_mode(socket: socket) -> OperationMode:
    """
    Gets the current operation mode of the switch.
    """
    returned_values = get_command_base(
        socket,
        __TARGET_MODE_OPERATION,
        __VALUE_PATTERNS_MODE_OPERATION,
        __VALUE_CONVERTERS_MODE_OPERATION,
    )
    return returned_values[0]


def set_operation_mode(socket: socket, op_mode: OperationMode) -> None:
    """
    Sets the operation mode of the switch.
    """
    set_command_base(
        socket,
        __TARGET_MODE_OPERATION,
        [op_mode],
        __VALUE_PATTERNS_MODE_OPERATION,
        __VALUE_CONVERTERS_MODE_OPERATION,
    )


__TARGET_MODE_AUTOSWITCH = "autosw"
__VALUE_CONVERTERS_MODE_AUTOSWITCH = [OnOffSwitch.from_string]
__VALUE_PATTERNS_MODE_AUTOSWITCH = [OnOffSwitch.pattern()]


def get_autoswitch_enabled(socket: socket) -> bool:
    """
    Gets whether automatic input switching is enabled on the switch.
    """
    returned_values = get_command_base(
        socket,
        __TARGET_MODE_AUTOSWITCH,
        __VALUE_PATTERNS_MODE_AUTOSWITCH,
        __VALUE_CONVERTERS_MODE_AUTOSWITCH,
    )
    return OnOffSwitch.to_bool(returned_values[0])


def set_autoswitch_enabled(socket: socket, enabled: bool) -> None:
    """
    Enables/disables automatic input switching on the switch.
    """
    set_command_base(
        socket,
        __TARGET_MODE_AUTOSWITCH,
        [OnOffSwitch.from_bool(enabled)],
        __VALUE_PATTERNS_MODE_AUTOSWITCH,
        __VALUE_CONVERTERS_MODE_AUTOSWITCH,
    )



#!/usr/bin/env python3

"""
Contains free functions for getting/setting VRROOM network settings.
"""

import socket
from typing import Any
from . import get_command_base, set_command_base
from .enums import OnOffSwitch


class IpAddressV4:
    """
    Defines valid IPv4 addresses.
    """

    def __init__(self, address: str) -> None:
        self.__address = address

    def __eq__(self, obj: Any) -> bool:
        return isinstance(obj, IpAddressV4) and (self.__address == obj.__address)

    def __str__(self) -> str:
        return self.__address

    @staticmethod
    def from_string(string: str) -> "IpAddressV4":
        """
        Converts a string into an instance of this class, if valid.
        """
        return IpAddressV4(string)

    @staticmethod
    def pattern() -> str:
        """
        Returns a valid regex pattern for this class.
        """
        return "((25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\.){3}(25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])"


__TARGET_IP_ADDRESS = "ipaddr"
__VALUE_CONVERTERS_IP_ADDRESS = [IpAddressV4.from_string]
__VALUE_PATTERNS_IP_ADDRESS = [IpAddressV4.pattern()]


def get_ip_address(socket: socket) -> IpAddressV4:
    """
    Gets the current IP address of the switch.
    """
    returned_values = get_command_base(
        socket,
        __TARGET_IP_ADDRESS,
        __VALUE_PATTERNS_IP_ADDRESS,
        __VALUE_CONVERTERS_IP_ADDRESS,
    )
    return returned_values[0]


def set_ip_address(socket: socket, address: IpAddressV4) -> None:
    """
    Sets the IP address of the switch.
    """
    set_command_base(
        socket,
        __TARGET_IP_ADDRESS,
        [address],
        __VALUE_PATTERNS_IP_ADDRESS,
        __VALUE_CONVERTERS_IP_ADDRESS,
    )


__TARGET_IP_NETWORK_MASK = "ipmask"


def get_ip_network_mask(socket: socket) -> IpAddressV4:
    """
    Gets the current IP network mask of the switch.
    """
    returned_values = get_command_base(
        socket,
        __TARGET_IP_NETWORK_MASK,
        __VALUE_PATTERNS_IP_ADDRESS,
        __VALUE_CONVERTERS_IP_ADDRESS,
    )
    return returned_values[0]


def set_ip_network_mask(socket: socket, address: IpAddressV4) -> None:
    """
    Sets the IP network mask of the switch.
    """
    set_command_base(
        socket,
        __TARGET_IP_NETWORK_MASK,
        [address],
        __VALUE_PATTERNS_IP_ADDRESS,
        __VALUE_CONVERTERS_IP_ADDRESS,
    )


__TARGET_IP_GATEWAY = "ipgw"


def get_ip_gateway(socket: socket) -> IpAddressV4:
    """
    Gets the current IP gateway of the switch.
    """
    returned_values = get_command_base(
        socket,
        __TARGET_IP_GATEWAY,
        __VALUE_PATTERNS_IP_ADDRESS,
        __VALUE_CONVERTERS_IP_ADDRESS,
    )
    return returned_values[0]


def set_ip_gateway(socket: socket, address: IpAddressV4) -> None:
    """
    Sets the IP gateway of the switch.
    """
    set_command_base(
        socket,
        __TARGET_IP_GATEWAY,
        [address],
        __VALUE_PATTERNS_IP_ADDRESS,
        __VALUE_CONVERTERS_IP_ADDRESS,
    )


__TARGET_DHCP_ONOFF = "dhcp"
__VALUE_CONVERTERS_ONOFF = [OnOffSwitch.from_string]
__VALUE_PATTERNS_ONOFF = [OnOffSwitch.pattern()]


def get_dhcp_enabled(socket: socket) -> bool:
    """
    Gets whether DHCP is enabled on the switch.
    """
    returned_values = get_command_base(
        socket,
        __TARGET_DHCP_ONOFF,
        __VALUE_PATTERNS_ONOFF,
        __VALUE_CONVERTERS_ONOFF,
    )
    return OnOffSwitch.to_bool(returned_values[0])


def set_dhcp_enabled(socket: socket, enabled: bool) -> None:
    """
    Enables/disables DHCP on the switch.
    """
    set_command_base(
        socket,
        __TARGET_DHCP_ONOFF,
        [OnOffSwitch.from_bool(enabled)],
        __VALUE_PATTERNS_ONOFF,
        __VALUE_CONVERTERS_ONOFF,
    )


__TARGET_IP_INTERRUPT_ONOFF = "ipinterrupt"


def get_ip_interrupts_enabled(socket: socket) -> bool:
    """
    Gets whether IP interrupts are enabled on the switch.
    """
    returned_values = get_command_base(
        socket,
        __TARGET_IP_INTERRUPT_ONOFF,
        __VALUE_PATTERNS_ONOFF,
        __VALUE_CONVERTERS_ONOFF,
    )
    return OnOffSwitch.to_bool(returned_values[0])


def set_ip_interrupts_enabled(socket: socket, enabled: bool) -> None:
    """
    Enables/disables IP interrupts on the switch.
    """
    set_command_base(
        socket,
        __TARGET_IP_INTERRUPT_ONOFF,
        [OnOffSwitch.from_bool(enabled)],
        __VALUE_PATTERNS_ONOFF,
        __VALUE_CONVERTERS_ONOFF,
    )


class TcpPort:
    """
    Defines valid TCP ports.
    """

    def __init__(self, port: int) -> None:
        self.__port = port

    def __eq__(self, obj: Any) -> bool:
        return isinstance(obj, TcpPort) and (self.__port == obj.__port)

    def __str__(self) -> str:
        return str(self.__port)

    @staticmethod
    def from_string(string: str) -> "TcpPort":
        """
        Converts a string into an instance of this class, if valid.
        """
        return TcpPort(int(string))

    @staticmethod
    def pattern() -> str:
        """
        Returns a valid regex pattern for this class.
        """
        return "([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])"


__TARGET_TCP_PORT = "tcpport"
__VALUE_CONVERTERS_TCP_PORT = [TcpPort.from_string]
__VALUE_PATTERNS_TCP_PORT = [TcpPort.pattern()]


def get_tcp_port(socket: socket) -> TcpPort:
    """
    Gets what TCP port is being used for commands on the switch.
    """
    returned_values = get_command_base(
        socket,
        __TARGET_TCP_PORT,
        __VALUE_PATTERNS_TCP_PORT,
        __VALUE_CONVERTERS_TCP_PORT,
    )
    return returned_values[0]


def set_tcp_port(socket: socket, port: TcpPort) -> None:
    """
    Sets what TCP port is being used for commands on the switch.
    """
    set_command_base(
        socket,
        __TARGET_TCP_PORT,
        [port],
        __VALUE_PATTERNS_TCP_PORT,
        __VALUE_CONVERTERS_TCP_PORT,
    )



#!/usr/bin/env python3

"""
Contains class to encode/decode VRROOM commands for socket recv/send.
"""

import re
from typing import Any, ByteString, Callable, List


class Codec:
    """
    Encodes/decodes VRROOM commands for socket recv/send.
    """

    __COMMAND_TERMINATOR = "\n"

    @staticmethod
    def decode_response(
        response: ByteString,
        target: str,
        value_patterns: List[str],
        value_converters: List[Callable],
    ) -> List[Any]:
        """
        Decodes a raw bytestring respons from the VRROOM switch.

        Returns a string with the terminators removed.
        """
        response_raw = Codec.decode_response_raw(response)

        values_pattern = " ".join(value_patterns)
        response_pattern = f"{target} (?P<values>{values_pattern})"
        response_match = re.match(response_pattern, response_raw)
        if response_match is None:
            raise ValueError(
                f"Unable to parse response '{response}' from VRROOM command!"
            )

        response_value_strs = response_match.groupdict()["values"].split(" ")
        converter_response_pairs = zip(value_converters, response_value_strs)
        return [converter(value) for converter, value in converter_response_pairs]

    @staticmethod
    def decode_response_raw(response: ByteString) -> str:
        """
        Decodes a raw bytestring response from the VRROOM switch.

        Returns a string with the terminators removed.
        """
        # Match responses ending in newline, regardless of carriage return beforehand
        return re.match("([^\r\n]*)\r?\n", response.decode()).group(1)

    @staticmethod
    def encode_command_get(target: str) -> ByteString:
        """
        Encodes a command to get a value for the given target.

        Returns the bytestring for the command.
        """
        raw_command = f"get {target}\n"

        return Codec.encode_command_raw(raw_command)

    @staticmethod
    def encode_command_raw(command: str) -> ByteString:
        """
        Encodes a raw string command for the VRROOM switch.

        Returns a byte string with the correct terminators.
        """
        if command[-1:] != Codec.__COMMAND_TERMINATOR:
            command += Codec.__COMMAND_TERMINATOR
        return command.encode()

    @staticmethod
    def encode_command_set(target: str, values: List[Any]) -> ByteString:
        """
        Encodes a command to set a target to the given value(s).

        Returns the bytestring for the command.
        """
        values_str = " ".join([str(val) for val in values])
        raw_command = f"set {target} {values_str}"

        return Codec.encode_command_raw(raw_command)



#!/usr/bin/env python3

"""
Contains enumeration classes that are used across multiple commands.
"""

from enum import Enum


class OnOffSwitch(Enum):
    """
    Enumerates the values for a VRROOM on/off switch.
    """

    OFF = "off"
    ON = "on"

    def __str__(self) -> str:
        return str(self.value)

    @staticmethod
    def from_bool(on: bool) -> "OnOffSwitch":
        """
        Converts a boolean into a value of this enumeration, if possible.
        """
        if on:
            return OnOffSwitch.ON
        return OnOffSwitch.OFF

    @staticmethod
    def from_string(string: str) -> "OnOffSwitch":
        """
        Converts a string into a value of this enumeration, if possible.
        """
        return OnOffSwitch(string)

    @staticmethod
    def pattern() -> str:
        """
        Returns a regex pattern that matches values of this enumeration.
        """
        return f"(?:{OnOffSwitch.OFF}|{OnOffSwitch.ON})"

    @staticmethod
    def to_bool(on_off: "OnOffSwitch") -> bool:
        """
        Converts a string to a bool via this enumeration, if possible.
        """
        if on_off == OnOffSwitch.ON:
            return True
        return False



#!/usr/bin/env python3

import socket
from typing import Any, Callable, List
from ..codec import Codec

"""
Contains base free functions used by specific getter/setter commands.
"""

DEFAULT_RECEIVE_BUFFER_SIZE = 256


def get_command_base(
    socket: socket.socket,
    target: str,
    value_patterns: List[str],
    value_converters: List[Callable],
) -> List[Any]:
    """
    Base VRROOM command function used by specific getter commands.
    """
    socket.sendall(Codec.encode_command_get(target))
    response = socket.recv(DEFAULT_RECEIVE_BUFFER_SIZE)
    return Codec.decode_response(response, target, value_patterns, value_converters)


def set_command_base(
    socket: socket.socket,
    target: str,
    desired_values: List[Any],
    value_patterns: List[str],
    value_converters: List[Callable],
) -> None:
    """
    Base VRROOM command function used by specific setter commands.

    Raises ValueError if the returned values are different than the desired values.
    """
    socket.sendall(Codec.encode_command_set(target, desired_values))
    response = socket.recv(DEFAULT_RECEIVE_BUFFER_SIZE)

    returned_values = Codec.decode_response(
        response, target, value_patterns, value_converters
    )
    for returned, desired in zip(returned_values, desired_values):
        if returned != desired:
            returned_type = type(returned).__name__
            raise ValueError(
                f"Returned {returned_type} '{returned}' was different than desired ({desired})!"
            )



#!/usr/bin/env python3

"""
Contains free functions for getting/setting VRROOM inputs.
"""

from enum import IntEnum
import socket
from typing import List
from . import get_command_base, set_command_base


class Input(IntEnum):
    """
    Enumerates the selectable inputs on the VRROOM switch.
    """

    RX0 = 0
    RX1 = 1
    RX2 = 2
    RX3 = 3
    FOLLOW = 4

    def __str__(self):
        return str(self.value)

    @staticmethod
    def from_string(string: str) -> "Input":
        """
        Converts a string into a value of this enumeration, if possible.
        """
        return Input(int(string))

    @staticmethod
    def pattern() -> str:
        """
        Returns a regex pattern that matches values of this enumeration.
        """
        return f"[{Input.RX0}-{Input.FOLLOW}]"


__TARGET_SELECTED_INPUTS = "insel"
__VALUE_CONVERTERS_SELECTED_INPUTS = [Input.from_string, Input.from_string]
__VALUE_PATTERNS_SELECTED_INPUTS = [Input.pattern(), Input.pattern()]


def get_selected_inputs(socket: socket.socket) -> List[Input]:
    """
    Gets the currently selected inputs of the switch.
    """
    return get_command_base(
        socket,
        __TARGET_SELECTED_INPUTS,
        __VALUE_PATTERNS_SELECTED_INPUTS,
        __VALUE_CONVERTERS_SELECTED_INPUTS,
    )


def set_selected_inputs(
    socket: socket.socket, input_tx0: Input, input_tx1: Input
) -> None:
    """
    Sets the currently selected inputs of the switch.
    """
    set_command_base(
        socket,
        __TARGET_SELECTED_INPUTS,
        [input_tx0, input_tx1],
        __VALUE_PATTERNS_SELECTED_INPUTS,
        __VALUE_CONVERTERS_SELECTED_INPUTS,
    )


__TARGET_SELECTED_INPUT_TX0 = "inseltx0"
__TARGET_SELECTED_INPUT_TX1 = "inseltx1"
__VALUE_CONVERTERS_SELECTED_INPUT_TXN = [Input.from_string]
__VALUE_PATTERNS_SELECTED_INPUT_TXN = [Input.pattern()]


def get_selected_input_tx0(socket: socket.socket) -> Input:
    """
    Gets the currently selected input for output TX0.
    """
    returned_values = get_command_base(
        socket,
        __TARGET_SELECTED_INPUT_TX0,
        __VALUE_PATTERNS_SELECTED_INPUT_TXN,
        __VALUE_CONVERTERS_SELECTED_INPUT_TXN,
    )
    return returned_values[0]


def get_selected_input_tx1(socket: socket.socket) -> Input:
    """
    Gets the currently selected input for output TX1.
    """
    returned_values = get_command_base(
        socket,
        __TARGET_SELECTED_INPUT_TX1,
        __VALUE_PATTERNS_SELECTED_INPUT_TXN,
        __VALUE_CONVERTERS_SELECTED_INPUT_TXN,
    )
    return returned_values[0]


def set_selected_input_tx0(socket: socket.socket, input: Input) -> None:
    """
    Sets the currently selected input for output TX0.
    """
    set_command_base(
        socket,
        __TARGET_SELECTED_INPUT_TX0,
        [input],
        __VALUE_PATTERNS_SELECTED_INPUT_TXN,
        __VALUE_CONVERTERS_SELECTED_INPUT_TXN,
    )


def set_selected_input_tx1(socket: socket.socket, input: Input) -> None:
    """
    Sets the currently selected input for output TX1.
    """
    set_command_base(
        socket,
        __TARGET_SELECTED_INPUT_TX1,
        [input],
        __VALUE_PATTERNS_SELECTED_INPUT_TXN,
        __VALUE_CONVERTERS_SELECTED_INPUT_TXN,
    )
